/*
 * Um exemplo minimo de implementacao do algoritmo RSA em Java
 * Por Marcio H. C. d'Avila <mhavila@inet.com.br>, 2001
 */
import java.math.BigInteger;
import java.security.SecureRandom;

public class Rsa
{
  private BigInteger n, d, e;

  public Rsa(int bitlen)
  {
    SecureRandom r = new SecureRandom();
    BigInteger p = new BigInteger(bitlen / 2, 100, r);
    BigInteger q = new BigInteger(bitlen / 2, 100, r);

    n = p.multiply(q);
    BigInteger fi_n = p.subtract(BigInteger.ONE).multiply(q.subtract(BigInteger.ONE));

    e = new BigInteger("3");
    while(fi_n.gcd(e).intValue() > 1) e = e.add(new BigInteger("2"));

    d = e.modInverse(fi_n);
  }

  public BigInteger encrypt(BigInteger message)
  {
    return message.modPow(e, n);
  }

  public BigInteger decrypt(BigInteger message)
  {
    return message.modPow(d, n);
  }

  static public void main(String [] args)
  {
    System.out.println("Inicializando RSA...");
    Rsa rsa = new Rsa(1024);

    BigInteger M = new BigInteger("123456789054321");
    System.out.println("M = " + M);

    System.out.println("Encriptando e Decriptando...");
    BigInteger C = rsa.encrypt(M);
    System.out.println("C = " + C);
    BigInteger m = rsa.decrypt(C);
    System.out.println("m = " + m);
  }
} // class Rsa

